#!/home/moerman/cadbin/perl -w
# Perl script to get PWI text
#
  my $help_text = <<HELP;               # start of help text
# $0, Kees Moerman (c) March 2010
#
# Syntax: pwi_read.pl [ -o <outputfile> ] <inputfile>
# Where:
#   -?  displays this help message (and quits)
#   -o  alternate output file location/name
# This small Perl utility reads a pwi file, and extracts the text strings
# in it into a plain ASCII text file. Drawings are nog converted.
# No intelligence, just an attempt to extract the plain text content.
# Did not check accented characters yet.
# Output file is same as input file, with the .pwi extension replaced by .txt
#
HELP
#
# Change list:
#   C.M. Moerman, 12 March 2010 Original first version
#
# To do:
#

use warnings;                       # comparable to -w option
use strict;                         # don't forget to prefix vars with my

$help_text =~ s/^# ?//gm;           # clean up comment characters
if ( !defined $ARGV[0]              # no arguments or question for help?
  || (defined $ARGV[0] && ($ARGV[0] eq "-?" || $ARGV[0] eq "-h"  )))
{   print $help_text;
    exit 1;
}

use Getopt::Std;                    # -o <filename>, -d options
our ($opt_o, $opt_d) = ('', 0);
getopts('do:');

my $inputfile = shift;
die "Can not read '$inputfile'" if(! -f $inputfile);

if($opt_o eq '')                    # create output file name (if not specified)
{
    $opt_o = $inputfile;
    $opt_o =~ s/\.pwi$/.txt/i;      # replace trailing .pwi by .txt
}

# read file (binary format)
sub slurp_file {                    # (file name) -> string
    my $file =  shift || die "No file name given for slurp_file";

    local $/;
    undef $/;                       # slurp mode

    return undef if ( !-f $file);   # no such file -> empty string

    open (EXTRACT_FILENAME, $file)  # open file for output
    || die "Can't open file '$file' ($!)\n";
    binmode EXTRACT_FILENAME;
    my $contents = <EXTRACT_FILENAME> || "";
    close EXTRACT_FILENAME;

    return $contents;
}

my $inputstring = slurp_file($inputfile);
my $outputstring = "";              # location to build up output file


sub hexdump {   # (bytes, name)     # display data read in hex for debugging
    my $i;

    print STDERR "    Hexdump of $_[1]";
    foreach $i (0..length($_[0])-1)
    {
        print  STDERR "\n      " if( ($i & 0x0F) == 0); # new line every 16
        printf STDERR "%02x ", ord(substr($_[0], $i, 1));
    }
    print STDERR "\n";
}

sub check_endmarker                 # search for end-of-string marker
{
    my $start = shift;
    if(substr($inputstring, $start, 3) eq "\xC4\x00\x00")
    { return 1; }
    return 0;
}

sub scan_back                       # search backwards to start of string
{
    my $end = -2 + shift;
    while ((substr($inputstring, $end, 2) ne "\x0A\x00")
      &&   (substr($inputstring, $end, 2) ne "\x01\x00"))
    { $end--;
      last if $end == 0;
    }
    return $end?$end+2:0;
}

sub remove_markup       # remove markup indicating e.g. underline for url
{
    my $string = shift;
    $string =~ s/\xEA\x01\xE7\x06\x00//g;   # start hyperlink
    $string =~ s/\xEA\x00\xE7\x00\x00//g;   # end hyperlink
    $string =~ s/\xC1\xA0/ /g;              # ??? space ???
    return $string;
}

for my $i (0..(-3+length $inputstring) )
{
    if(check_endmarker($i))
    {
      my $start = scan_back($i);
      if($start)
      {
        my $string = remove_markup(substr($inputstring, $start, $i-$start));
        $outputstring .= "$string\n"
          unless ($string eq '') && ($outputstring eq '');
        printf "%04X: %s\n", $i, $string if $opt_d;
      }
    }
}
                                    # write result to file
open (FILE, ">$opt_o")              # open file for output
|| die "Can't open file '$opt_o' ($!)\n";
print FILE $outputstring;
close FILE;

# end of script

